//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/audio/audioinfo.c,v $
// $Id: audioinfo.c,v 1.1 2005/11/03 21:56:08 ah Exp $ 
//
// Description: print information about specified audio file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: audioinfo.c,v 1.1 2005/11/03 21:56:08 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <cstdlib>
#ifdef NO_UNISTD
#   include <getopt.h>
#else
#   include <unistd.h>
#endif
#include <string>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <fstream>
USING_NAMESPACE(std)

#include <AH/Error/Class.h>
#include <AH/IO/AudioFile.h>
USING_NAMESPACE(AH)

// --- local constants ---

static const char* cmdLineOptions = "hv";
static const char* version = "0.1";

// --- local functions ---


static void usage()
{
    cerr << "Usage: audioinfo [-" << cmdLineOptions << "] file [file, ...]\n"
	 << "       print information about audio file(s).\n"
	 << "  options:\n"
	 << "    h   - print this help message\n"
	 << "    v   - print version" << endl;
} // end of usage()


static void print_version()
{
    cerr << "audioinfo Vs " << version << " " 
	 << "(c) 2005 Andreas Hnnebeck\n"
	 << "Licensed under the terms of the GNU General Public License.\n"
	 << "Report bugs to andreas@huennebeck-online.de\n"
	 << "http://www.huennebeck-online.de"
	 << endl;
} // end of print_version()


// --- global functions ---


int main(int argc, char * const argv[])
{
    // loop through command line options:
    int option;
    while ((option = getopt(argc, argv, cmdLineOptions)) != -1)
    {
	switch (option) 
	{
	case 'h': usage(); exit(0);
	case 'v': print_version(); exit(0);
	default : usage(); exit(1);
	}
    }
    if (optind >= argc) 
    {
	cerr << "Error: missing file" << endl;
	usage();
	exit(1);
    }

    // loop through the remaining arguments: these are hopefully audio files
    unsigned int errnum = 0;
    for (int i=optind; i < argc; i++)
    {
	const char* filename = argv[i];
	try
	{
	    // create a audio file object and let it print some info
	    IOAudioFile<int32> audio(filename, false);
	    cout << audio.getInfo();
	}
	catch(ErrorClass& error)
	{
	    errnum++;
	    cerr << "Error processing " << filename << ": " << error.getMsg() 
		 << endl;
	}
    }

    exit (errnum ? 1 : 0);
    return  (errnum ? 1 : 0); // to keep MSVC quiet
} // end of main()


// --- end of file ---
