//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/ft/RedSpec.h,v $
// $Id: RedSpec.h,v 1.5 2003/03/12 00:55:19 ah Exp $ 
//
//---------------------------------------------------------------------------

#ifndef AH_FT_REDUCED_SPECTRUM
#define AH_FT_REDUCED_SPECTRUM

const char FtReducedSpectrum_CVSIDd[] = "$Id: RedSpec.h,v 1.5 2003/03/12 00:55:19 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h> // always the first include
#include <AH/Config/Types.h>
#include <AH/Etech/Types.h>
USING_NAMESPACE(AH)

// --- constants ---

// --- classes ---

class ReducedSpectrum
{
 public:
    ReducedSpectrum(unsigned int in_size, unsigned int out_size, 
		    double max_freq, unsigned int debug = 0);
    ~ReducedSpectrum();
    uint32 process(Cplx* spectrum, bool is_linear);

    const double* getMeanAbs() const { return mean_abs_spectrum; }
    const double* getMeanPhi() const { return mean_phi_spectrum; }
    const double* getMaxAbs() const { return max_abs_spectrum; }
    const double* getMaxPhi() const { return max_phi_spectrum; }
    const double* getMinAbs() const { return min_abs_spectrum; }
    const double* getMinPhi() const { return min_phi_spectrum; }
    const double* getFreq() const { return frequencies; }

    double getMeanAbsVal(uint32 i) const { return mean_abs_spectrum[i]; }
    double getMaxAbsVal(uint32 i) const  { return max_abs_spectrum[i]; }
    double getMinAbsVal(uint32 i) const  { return min_abs_spectrum[i]; }
    double getMeanPhiVal(uint32 i) const { return mean_phi_spectrum[i]; }
    double getMaxPhiVal(uint32 i) const  { return max_phi_spectrum[i]; }
    double getMinPhiVal(uint32 i) const  { return min_phi_spectrum[i]; }
    double getFreqVal(uint32 i) const    { return frequencies[i]; }

    void addMeanAbsVal(uint32 i, double val) { mean_abs_spectrum[i] += val; }
    void addMaxAbsVal(uint32 i, double val) { max_abs_spectrum[i] += val; }
    void addMinAbsVal(uint32 i, double val) { min_abs_spectrum[i] += val; }

 private:
    unsigned int inum;
    unsigned int onum;
    double f_max;
    double f_inc;
    unsigned int debon;

    double* mean_abs_spectrum;
    double* mean_phi_spectrum;
    double* max_abs_spectrum;
    double* max_phi_spectrum;
    double* min_abs_spectrum;
    double* min_phi_spectrum;
    double* frequencies;
};

#endif // !AH_FT_REDUCED_SPECTRUM
// --- end of file ---
