//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/ft/sweep.c,v $
// $Id: sweep.c,v 1.1 2002/11/01 18:16:38 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char sweep_CVSID[] = "$Id: sweep.c,v 1.1 2002/11/01 18:16:38 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "sweep.h"

#include <math.h>
#include <drfftw.h> 
USING_NAMESPACE(std)

#include <AH/Etech/Types.h>
#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)

#include "wisdom.h"

// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

SweepSpectrum::SweepSpectrum 
(
    unsigned int num,
    double f1,
    double f2,
    double dur,
    bool lin = false,
    unsigned int debug
)
    : isize(num), dsize((isize+1)/2),
      begin(f1), end(f2), duration(dur), option_linear(lin),
      debon(debug), data(new Cplx[dsize+1])
{
    // create data array:
    if (!this->data) throw ErrorSysOutOfMem();
    this->calculate();
}


SweepSpectrum::~SweepSpectrum()
{
    if (this->data) delete[] this->data;
}


const Cplx& SweepSpectrum::getPoint(unsigned int num) const
{
    if (this->data && (num < this->dsize)) return this->data[num];
    throw ErrorInternBadArg();
}


void SweepSpectrum::calculate()
{
    // create input and output buffers for FFT:
    double* din = new  double[this->isize];
    if (!din) throw ErrorSysOutOfMem();
    double* dout = new double[this->isize];
    if (!dout)  throw ErrorSysOutOfMem();

    // fill input buffer with sweep:
    this->makeSweep(din);

    // make inplace fft
    FtWisdom wis(this->debon);
    rfftw_one(wis.getPlan(this->isize), din, dout);

    // fill complex spectrom data array
    data[0] = Cplx(dout[0], 0);
    unsigned int i;
    for (i = 1; i < dsize; ++i)
	data[i] = Cplx(dout[i], dout[isize-i]);
    if (isize % 2)
	data[i] = Cplx(dout[isize/2], 0);

    delete[] din;
    delete[] dout;
}


void SweepSpectrum::makeSweep(double* sweep)
{
    double delta = (begin < end ? end - begin : begin - end);

    // create sweep input array:
    const double zwo_pi = 2.0 * M_PI;
    unsigned int i = 0;
    for (double t = 0; t < rate * duration; ++t, ++i)
    {
	double f;
	if (option_linear)
	{
	    if ( begin < end )
		f = begin + (delta * t / (rate*duration));
	    else
		f = begin - (delta * t / (rate*duration));
	}
	else
	{
	    if ( begin < end )
		f = begin * pow (1.0*end/begin, t*(1.0/rate/duration));
	    else
		f = begin / pow (1.0*begin/end, t*(1.0/rate/duration));
	}

	double phi = zwo_pi * (f/rate);
	if (phi > zwo_pi) phi -= zwo_pi;
//	short sample = (int16) (32767.0 * sin (phi));
	sweep[i] = 32767.0 * sin (phi);
    }

    // add zero filling  if nedded:
    for (; i < isize; ++i)
	sweep[i] = 0;
}


// --- end of file ---
