//---------------------------------------------------------------------------
// Copyright 2003 Andreas Hnnebeck
// Copyright 2003 Thomas Ludwig (fast algorithm in SigMls::addSignal())
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigMls.c,v $
// $Id: SigMls.c,v 1.8 2004/02/05 21:30:47 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char sweep_CVSID[] = "$Id: SigMls.c,v 1.8 2004/02/05 21:30:47 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigMls.h"

#include <math.h>
#include <time.h>
#include <unistd.h>
USING_NAMESPACE(std)

#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)


// --- local constants ---

#define TIMETRACE 0

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

SignalMls::SignalMls
(
    std::vector<unsigned int>& taps,
    double amplitude,
    unsigned int samplerate,
    unsigned int debug
)
    : SignalBase(amplitude, samplerate, debug),
      numElem(0), polynomial(0)
{
    // convert taps into polynomial, thereby looking for the highest tap
    for (unsigned int t = 0; t < taps.size(); ++t)
    {
	if (debon > 2) cerr << "MLS: tap[" << t << "] = " << taps[t] << endl; 
	polynomial |= (1 << taps[t]-1);
	if (taps[t] > numElem) numElem = taps[t];
    }
}


/* Thomas Ludwig 2003-04-18 */
/* LFSR in galois-configuration. pp's Funktion als Vorbild */
inline static unsigned int lfsr_galois 
(
    unsigned int polynomial,
    unsigned int state
)
{
    return ((state >> 1) ^ ((state & 1) * polynomial));
}


/* adapted from Thomas Ludwig's algorithm as of 2003-04-27 */
void SignalMls::addSignal
(
    double* data,
    unsigned int num
)
{
    if (!data) throw ErrorInternNullPtrArg("SignalMls::addSignal");
    if (!num) throw ErrorInternBadArg("SignalMls::addSignal");

    //-------------------------------------
    // copy MLS sequence into output data:
    //-------------------------------------
    if (debon > 1) cerr << "MLS: order = " << numElem << endl; 
    if (debon > 1) cerr << "MLS: polynomial = " << polynomial << endl; 
    unsigned int seed = 1;
    unsigned int state = seed;
    unsigned int i = 0;
    do
    {
	if (state & 1) data[i] -= maxAmpl;
	else           data[i] += maxAmpl;
	state = lfsr_galois(polynomial, state);
    } while (++i < num);
}


#if 0
// This is the original (slow) code by Andreas Hnnebeck
void SignalMls::addSignal
(
    double* data,
    unsigned int num
)
{
    if (!data) throw ErrorInternNullPtrArg("SignalMls::addSignal");
    if (!num) throw ErrorInternBadArg("SignalMls::addSignal");

#if TIMETRACE
    const double msec_per_clock = 1000.0 / CLOCKS_PER_SEC;
    double start_time = clock() * msec_per_clock;
#endif

    // get length of MLS:
    unsigned int mlslen = (1 << numElem) - 1;
    if (mlslen > num) mlslen = num;

    bool* mlsdata = new bool[mlslen];
    if (!mlsdata) throw ErrorSysOutOfMem();

    //-----------------------------------------
    // generate MLS sequence (1 cycle maximum)
    //-----------------------------------------
    unsigned int start = 0;
    for (unsigned int i = 0; i < mlslen; ++i)
    {
	// get value of last register value:
	unsigned int last = (start + numElem - 1) % numElem;
	mlsdata[i] = shiftReg[last];

	// calculate the new start value
	unsigned int tapidx = (start + tap[0] - 1) % numElem;
	bool newval = shiftReg[tapidx];
	for (unsigned int k = 1; k < tap.size(); ++k)
	{
	    tapidx = (start + tap[k] - 1) % numElem;
	    newval = (newval != shiftReg[tapidx]); // xor
	}

	// execute the shift and insert the new value:
	// - instead of shifting we move the start pointer
	if (start > 0) --start;
	else start = numElem - 1;
	shiftReg[start] = newval;
    }

#if TIMETRACE
    double mls_time = clock() * msec_per_clock;
#endif

    //-------------------------------------
    // copy MLS sequence into output data:
    //-------------------------------------
    if (reversed)
    {
	if (debon) cerr << "Attention: generating inverse MLS" << endl;
	for (unsigned int i = 0, m = mlslen - 1; i < num; ++i)
	{
	    if (mlsdata[m])  data[i] += maxAmpl;
	    else             data[i] -= maxAmpl;
	    if (0 == m) m = mlslen - 1;
	    else        --m;
	}
    }
    else
    {
	for (unsigned int i = 0, m = 0; i < num; ++i, ++m)
	{
	    if (m == mlslen) m = 0;
	    if (mlsdata[m])  data[i] += maxAmpl;
	    else             data[i] -= maxAmpl;
	}
    }

    delete[] mlsdata;
#if TIMETRACE
    double copy_time = clock() * msec_per_clock;
    cerr << "time for MLS: " << mls_time - start_time
	 << " ms, time for copy: " << copy_time - start_time << " ms"
	 << endl;
#endif
}
#endif // original slow code by Andreas Hnnebeck

// --- end of file ---












