//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigPinkNoise.c,v $
// $Id: SigPinkNoise.c,v 1.1 2002/12/07 02:38:20 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char sweep_CVSID[] = "$Id: SigPinkNoise.c,v 1.1 2002/12/07 02:38:20 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigPinkNoise.h"

#include <math.h>
#include <drfftw.h> 
USING_NAMESPACE(std)

#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
#include <AH/Math/Ft/Wisdom.h>
USING_NAMESPACE(AH)

#include "SigWhiteNoise.h"

// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

SignalPinkNoise::SignalPinkNoise
(
    double amplitude,
    unsigned int samplerate, 
    unsigned int seed,
    unsigned int debug
)
    : SignalBase(amplitude, samplerate, debug)
{
    srandom(seed);
}

void SignalPinkNoise::addSignal
(
    double* data,
    unsigned int num
)
{
    if (debon) cerr << "SPN::addSignal: started" << endl;
    if (!data) throw ErrorInternNullPtrArg("SignalPinkNoise::addSignal");
    if (!num) throw ErrorInternBadArg("SignalPinkNoise::addSignal");

    fftw_real* td = new fftw_real[num];      // input samples
    if (!td) throw ErrorSysOutOfMem("SignalPinkNoise::addSignal");
    fftw_real* spectrum = new fftw_real[num];    // output samples
    if (!spectrum)
    {
	delete[] td;
	throw ErrorSysOutOfMem("SignalPinkNoise::addSignal");
    }

    // fill td with falling spectrum:
#if 0
    for (unsigned int i = 0; i < num/2; ++i)
	td[i] = maxAmpl;
    for (unsigned int i = num/2; i < num; ++i)
	td[i] = 0;
#else
    for (unsigned int i = 0; i < num; ++i)
	td[i] = 0;
    SignalWhiteNoise wnoise(maxAmpl, debon);
    wnoise.addSignal(td, num);
#endif

    // get plan for FFT:
    MathFtWisdom wis(debon);
    if (debon) cerr << "SPN::addSignal: get plan for " << num
		    << " points" << endl;
    rfftw_plan p = wis.getPlan(num, FFTW_FORWARD);

    // run FFT:
    if (debon) cerr << "SPN::addSignal: run FFT" << endl;
    rfftw_one(p, td, spectrum);

    // convert white noise spectrum to pink noise:
    if (debon) cerr << "convert white noise spectrum to pink noise" << endl;
    for (unsigned int i = 1; i < num/2; ++i)
	spectrum[i] /= sqrt(i);

    // get plan for inverse FFT:
    if (debon) cerr << "SPN::addSignal: get plan for " << num
		    << " points" << endl;
    p = wis.getPlan(num, FFTW_BACKWARD);

    // run inverse FFT:
    if (debon) cerr << "SPN::addSignal: run inverse FFT" << endl;
    rfftw_one(p, spectrum, td);
    delete[] spectrum; spectrum = 0;

    // correct amplitude:
    if (debon) cerr << "SPN::addSignal: correct amplitude" << endl;
    double corr = 1;
    double max = 0;
    for (unsigned int i = 0; i < num; ++i)
    {
	double cur = fabs(td[i]);
	if (cur > max) max = cur;
    }
    if (max > 0) corr = maxAmpl / max;
    else cerr << "SPN::addSignal: warning - no maximum found"
	      << endl;
    if (debon > 2) cerr << "max=" << max << " corr=" << corr << endl;

    // copy td into data:
    if (debon > 1) cerr << "SPN::addSignal: add to data" << endl;
    for (unsigned int i = 0; i < num; ++i)
    {
	data[i] += corr * td[i];
	if (debon > 3) cerr << i << " " << data[i] << endl;;
    }

    delete[] td; td = 0;
}

// --- end of file ---











