//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigPinkNoise.c,v $
// $Id: SigPinkNoise.c,v 1.1 2002/12/07 02:38:20 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char sweep_CVSID[] = "$Id: SigPinkNoise.c,v 1.1 2002/12/07 02:38:20 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigPinkNoise.h"

#include <math.h>
#include <drfftw.h> 
USING_NAMESPACE(std)

#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
#include <AH/Math/Ft/Wisdom.h>
#include <AH/Math/Dither.h>
USING_NAMESPACE(AH)


// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

SignalPinkNoise::SignalPinkNoise
(
    double amplitude,
    unsigned int samplerate, 
    unsigned int seed,
    unsigned int debug
)
    : SignalBase(amplitude, samplerate, debug)
{
    srandom(seed);
}

void SignalPinkNoise::addSignal
(
    double* data,
    unsigned int num
)
{
    if (debon) cerr << "SignalPinkNoise::addSignal: started" << endl;
    if (!data) throw ErrorInternNullPtrArg("SignalPinkNoise::addSignal");
    if (!num) throw ErrorInternBadArg("SignalPinkNoise::addSignal");

    fftw_real* din = new fftw_real[num];      // input samples
    if (!din) throw ErrorSysOutOfMem("SignalPinkNoise::addSignal");
    fftw_real* dout = new fftw_real[num];    // output samples
    if (!dout)
    {
	delete[] din;
	throw ErrorSysOutOfMem("SignalPinkNoise::addSignal");
    }

    // fill din with falling spectrum:
#if 0
    for (unsigned int i = 0; i < num/2; ++i)
	din[i] = maxAmpl;
//	din[i] = maxAmpl / sqrt(i);
    for (unsigned int i = num/2; i < num; ++i)
	din[i] = 0;
#else
    MathDither dither(MathDither::RectAngle);
    // real part:
    for (unsigned int i = 0; i < num/2; ++i)
    {
	din[i] = maxAmpl; // + maxAmpl/10.0 * dither.getSignal();
	if (i) din[i] /= sqrt(i);
    }

#if 0
    const unsigned int limit = 50;
    unsigned int i1 =  num/2 / limit;
    unsigned int i2 =  num/2 - i1;
    for (unsigned int i = 0; i < i1; ++i)
	din[i] *= i * limit/num;
    for (unsigned int i = i2; i < num/2; ++i)
	din[i] *= (num/2 -i ) * limit/num;
#endif

    // imaginary part:
    for (unsigned int i = num/2; i < num; ++i)
	din[i] = 0;
#endif

    // get plan for inverse FFT:
    MathFtWisdom wis(debon);
    if (debon) cerr << "SignalPinkNoise::addSignal: get plan for " << num
		    << " points" << endl;
    rfftw_plan p = wis.getPlan(num, FFTW_BACKWARD);

    // run inverse FFT:
    if (debon) cerr << "SignalPinkNoise::addSignal: run inverse FFT" << endl;
    rfftw_one(p, din, dout);
    delete[] din; din = 0;
    dout[0] = 0;

    // correct amplitude:
    if (debon) cerr << "SignalPinkNoise::addSignal: correct amplitude" << endl;
    double corr = 1;
    double max = 0;
    for (unsigned int i = 0; i < num; ++i)
    {
	double cur = fabs(dout[i]);
	if (cur > max) max = cur;
    }
    if (max > 0) corr = maxAmpl / max;
    else cerr << "SignalPinkNoise::addSignal: warning - no maximum found"
	      << endl;
    if (debon > 2) cerr << "max=" << max << " corr=" << corr << endl;
    // copy td into data:
    if (debon > 1) cerr << "SignalPinkNoise::addSignal: add to data" << endl;
    for (unsigned int i = 0; i < num; ++i)
    {
	data[i] += corr * dout[i];
	if (debon > 3) cerr << i << " " << data[i] << endl;;
    }

    delete[] dout; dout = 0;
}

// --- end of file ---











