//---------------------------------------------------------------------------
// Copyright 2002 Andreas Huennebeck
// Sweep Algorithm: Copyright 2002 Patrick Piecha
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigRiaaSweep.c,v $
// $Id: SigRiaaSweep.c,v 1.1 2002/12/16 23:42:26 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char SigSweep_CVSID[] = "$Id: SigRiaaSweep.c,v 1.1 2002/12/16 23:42:26 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigRiaaSweep.h"

#include <math.h>
USING_NAMESPACE(std)

#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
#include <AH/Etech/Riaa.h>
USING_NAMESPACE(AH)


// --- local constants ---

#define AH_OPTIMIZE 1

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---


SignalRiaaSweep::SignalRiaaSweep
(
    double fbegin, 
    double fend,
    bool linear,
    double stime, 
    double amplitude,
    unsigned int samplerate,
    unsigned int debug
)
    : SignalBase(amplitude, samplerate, debug), 
      f1(fbegin), f2(fend), tim(stime), lin(linear)
{
    if (f1 < 0) throw ErrorInternBadArg("sweep start frequency");
    if (f2 < 0) throw ErrorInternBadArg("sweep end frequency");
    if (!lin)
    {
	if (f1 == 0) throw ErrorInternBadArg("sweep start frequency");
	if (f2 == 0) throw ErrorInternBadArg("sweep end frequency");
    }
}



void SignalRiaaSweep::addSignal
(
    double* data,
    unsigned int num
)
{
    if (!data) throw ErrorInternNullPtrArg("SignalRiaaSweep::addSignal");
    if (!num) throw ErrorInternBadArg("SignalRiaaSweep::addSignal");

    // determine amplitude factor:
    // the RIAA preemphasis has the highest amplitude at the highest 
    // frequency:
    double fmax = (f1 > f2 ? f1 : f2);
    double riaa_max = abs(RiaaPreEmphasis(fmax));
    double amplCorr = maxAmpl / riaa_max;

    // calculate the sweep:
    const double zwo_pi = 2.0 * M_PI;
    if (lin)
    {
	if (debon) cerr << "sweep: linear" << endl;
	const double delta = (f2 - f1) / (double)(num - 1);
	double phi = 0;
	for (unsigned int i = 0; i < num; ++i)
	{
	    double f = f1 + i * delta;
	    phi += zwo_pi * (f/sr);
	    while (phi > zwo_pi) phi -= zwo_pi;
	    data[i] += amplCorr * abs(RiaaPreEmphasis(f)) * sin (phi);
	}
    }
    else
    {
	if (debon) cerr << "sweep: logarithmic" << endl;
	const double tmpf1 = 1.0 / (double)(num - 1);
	const double tmpf2 = f2 / f1;
	double phi = 0;
	for (unsigned int i = 0; i < num; ++i)
	{
	    double f = f1 * pow (tmpf2, i * tmpf1);
	    phi += zwo_pi * (f/sr);
	    while (phi > zwo_pi) phi -= zwo_pi;
	    data[i] += amplCorr * abs(RiaaPreEmphasis(f)) * sin (phi);
	}
    }
}

// --- end of file ---











