//---------------------------------------------------------------------------
// Copyright 2002 Andreas Huennebeck
// Sweep Algorithm: Copyright 2002 Patrick Piecha
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigRndSweep.c,v $
// $Id: SigRndSweep.c,v 1.3 2004/04/21 20:41:06 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char SigSweep_CVSID[] = "$Id: SigRndSweep.c,v 1.3 2004/04/21 20:41:06 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigRndSweep.h"

#include <stdlib.h>
#include <math.h>
USING_NAMESPACE(std)

#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)


// --- local constants ---

#define AH_OPTIMIZE 1

// --- local data ---

// --- local classes ---

// --- local functions ---

#ifdef HAS_NO_LOG2
static inline double log2(double x)
{
    return log(x)/log((double)2);
}
#endif
// --- global functions ---


SignalRandomSweep::SignalRandomSweep
(
    double fbegin, 
    double fend,
    bool linear,
    double stime, 
    double amplitude,
    unsigned int samplerate,
    unsigned int debug
)
    : SignalBase(amplitude, samplerate, debug), 
      f1(fbegin), f2(fend), tim(stime), lin(linear)
{
    if (f1 < 0) throw ErrorInternBadArg("sweep start frequency");
    if (f2 < 0) throw ErrorInternBadArg("sweep end frequency");
    if (f1 == f2) throw ErrorInternBadArg("equal sweep start/end frequencies");
    if (!lin)
    {
	if (f1 == 0) throw ErrorInternBadArg("sweep start frequency");
	if (f2 == 0) throw ErrorInternBadArg("sweep end frequency");
    }
}



void SignalRandomSweep::addSignal
(
    double* data,
    unsigned int num
)
{
    if (!data) throw ErrorInternNullPtrArg("SignalRandomSweep::addSignal");
    if (!num) throw ErrorInternBadArg("SignalRandomSweep::addSignal");

    //-----------------------------------------------------------
    // we may change the frequency not faster than the highest
    // frequency:
    // The factor 10 has been found empirically to be the best
    // compromise:
    // - Lower factors deform the spectrum to a peak with an 
    //   attenuation of more than 6 dB at f1 and f2. 
    // - Higher factors are problematic for low frequencies and
    //   short play times.
    //-----------------------------------------------------------
    double fmax = (f2 > f1 ? f2 : f1);
    unsigned int N = 10 * (unsigned int)(sr / fmax);
    if (debon) cerr << "N=" << N << endl;

    double b;
    if (lin)
    {
	// determine shift factor:
	// random() returns 0..RAND_MAX which must be mapped into f1..f2
	b = (f2 - f1) / (double)RAND_MAX;
    }
    else
    {
	// determine factor:
	// random() returns 0..RAND_MAX which must be mapped into f1..f2
	// using f = a * 2^(i*b) with f(0) = f1 and f(RAND_MAX) = f2:
	//   (1) f(0) = a * 20 = a = f1
	//   (2) f(RM) = f1 * 2^(RM*b)  ==> b = log2(f2/f1) / RM
	b = log2(f2 / f1) / (double)RAND_MAX;
    }

    // calculate the random sweep:
    // based on Patrick Piecha's sinus sweep algorithm
    if (debon) cerr << "random sweep" << endl;
    const double zwo_pi = 2.0 * M_PI;
    double f = f1;
    double phi = 0;
    for (unsigned int i = 0; i < num; ++i)
    {
	if (0 == (i % N))
	{
	    // calculate next frequency:
	    if (lin) f = f1 + b * random();
	    else     f = f1 * pow(2.0, b * random());
	}
	phi += zwo_pi * (f/sr);
	while (phi > zwo_pi) phi -= zwo_pi;
	data[i] += maxAmpl * sin (phi);
    }
}

// --- end of file ---











