//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigTriangle.c,v $
// $Id: SigTriangle.c,v 1.1 2002/12/17 21:17:18 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char sweep_CVSID[] = "$Id: SigTriangle.c,v 1.1 2002/12/17 21:17:18 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigTriangle.h"

#include <math.h>
USING_NAMESPACE(std)

#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)


// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

void SignalTriangle::addSignal
(
    double* data,
    unsigned int num
)
{
    if (!data) throw ErrorInternNullPtrArg("SignalTriangle::addSignal");
    if (!num) throw ErrorInternBadArg("SignalTriangle::addSignal");

    const unsigned int tau = (unsigned int)(sr / freq);
    const unsigned int tau1 = tau/4;       // for performance reasons
    const unsigned int tau3 = tau - tau/4; // for performance reasons

    //--------------------------------------------------
    // x = i % tau   ==> 0 .. tau-1
    // y = a*x + b : a=(y2-y1)/(x2-x1) and b=y1-a*x1
    //
    // 1. quadrant:   y(0)        = 0,        y(tau/4-1)   = maxAmpl
    //   y = a1 * x + b1
    //   ==> a1 = maxAmpl / (tau/4 - 1)
    //       b1 = 0
    double a1 = maxAmpl / (double)(tau1-1);
    //
    // 2+3. quadrant: y(tau/4-1)  = maxAmpl,  y(tau*3/4-1) = -maxAmpl
    //   y = a23 * x + b23
    //   ==> a23 = -2 * maxAmpl / (tau/2) = -4 * maxAmpl / tau
    //       b23 = maxAmpl + 4 * maxAmpl/tau * (tau/4-1)
    //           = maxAmpl (1 + (4/tau * tau/4-1))
    //           = maxAmpl (2 - 4/tau)
    double a23 = -4.0 * maxAmpl / (double)tau;
    double b23 = maxAmpl * (2.0 - 4.0/(double)tau);
    //
    // 4. quadrant:   y(tau3/4-1) = -maxAmpl, y(tau-1)     = 0
    //   y = a4 * x + b4
    //   ==> a4 = maxAmpl / (tau/4) = 4 * maxAmpl / tau = -a23
    //       b4 = -maxAmpl - 4*maxAmpl/tau * tau3/4-1
    //          = -maxAmpl (1 + 4/tau * (tau*3/4 - 1))
    //           = maxAmpl (4 -4/tau)
    double a4 = -a23;
    double b4 = -maxAmpl * (4.0 - 4.0/(double)tau);
    //--------------------------------------------------
    for (unsigned int i = 0; i < num; ++i)
    {
	unsigned int x = i % tau;
	if (x < tau1)      // 1. quadrant
	    data[i] += a1 * (double)x ;
	else if (x < tau3) // 2+3. quadrant
	    data[i] += a23 * (double)x + b23;
	else               // 4. quadrant
	    data[i] += a4 * (double)x + b4;
    }
}

// --- end of file ---











