//---------------------------------------------------------------------------
// Copyright 2005 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// Additionally this source is licensed exclusively to the company
// Bruker BioSpin GmbH under the terms of the GNU Lesser General Public 
// License (LGPL). See the file 'COPYING.LIB' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/CUtil/SecureString.h,v $
// $Id: SecureString.h,v 1.2 2005/09/28 23:44:03 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef	AH_CUTIL_SECURE_STRING_H		// prevent multiple includes
#define AH_CUTIL_SECURE_STRING_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
USING_NAMESPACE(AH)

#include <sys/types.h>

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/** utilities for secure handling of C character arrays in C++ code.

    Class CUtilSecureString offers some functions to handle
    C style character arrays in a secure way. 

    All functions are static and inline and can be used without
    object.
 */
class CUtilSecureString
{
 public:
    /** copy character arrays.

       The strnCpy() function copies the string pointed to by 'src'
       (including the terminating `\0' character) to the array 
       pointed to by 'dest'. No more than 'n'-1 bytes of 'src' are
       copied, since 'dest' is guaranteed to be always NULL terminated.
       The character arrays may not overlap.
       @param dest pointer to the destination character array
       @param source pointer to the source character array
       @param n the size of the destination char array
       @return pointer to 'dest'
    */
    static char* strnCpy(char* dest, const char* src, size_t n);

    /** copy character arrays.

       The strCpy() function copies the string pointed to by 'src'
       (including the terminating `\0' character) to the array 'dest'.
       It is guaranteed that no more characters are written into 
       'dest' than 'dest' can keep, and it is guaranteed that 'dest'
       is always NULL terminated. The character arrays may not overlap.
       @param dest the destination character array (no pointer)
       @param source pointer to the source character array
       @return pointer to 'dest'
    */
    template<class CharArray>
	static char* strCpy(CharArray d, const char* src)
	{
	    return CUtilSecureString::strnCpy(
		d, src, sizeof(d)/sizeof(d[0]));
	}

}; // class CUtilSecureString


END_NAMESPACE

#endif // AH_CUTIL_SECURE_STRING_H

