//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Capacitor.h,v $
// $Id: Capacitor.h,v 1.8 2004/04/20 22:04:48 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_CAPACITOR_H
#define ETECH_CAPACITOR_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/FreqImp.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an ideal electrical capacitor
/**
 {\tt Capacitor} represents an ideal electrical capacitor which can be used both 
 in DC (direct current) and in AC (alternate current) applications.
 */
class Capacitor: public FreqImpedance
{
  private:
    Dbl Capacity;

  public:
    /// constructor
    Capacitor(Dbl c = 0.0) : FreqImpedance(), Capacity(c), frequency(-1) {}
//    Capacitor(const FreqImpedance& i) : FreqImpedance(i), frequency(-1) {}
    /// copy constructor
    Capacitor(const Capacitor& c) 
	: FreqImpedance(), Capacity(c.Capacity), frequency(-1) {}

    /// return capacity
    Dbl getCap() const { return Capacity; }

    /// overloaded operator + for serial circuit of two capacitors
    friend Capacitor operator + (const Capacitor &c1, const Capacitor &c2)
	{ return Capacitor(c1.Capacity * c2.Capacity / 
			   (c1.Capacity + c2.Capacity)); }

    /// overloaded operator || for parallel circuit of two capacitors
    friend Capacitor operator || (const Capacitor &c1, const Capacitor &c2)
	{ return Capacitor(c1.Capacity + c2.Capacity); }

    /// overloaded operator * for multiplication of {\tt Capacitor} * {\tt Dbl}
    friend Capacitor operator * (const Capacitor &c, Dbl k)
	{ return Capacitor(c.Capacity * k); }
    /// overloaded operator * for multiplication of{\t Dbl} * {\tt Capacitor}
    friend Capacitor operator * (Dbl k, const Capacitor &c)
	{ return Capacitor(k * c.Capacity); }

    /// overloaded operator / for division of {\tt Capacitor} / {\tt Dbl}
    friend Capacitor operator / (const Capacitor &c, Dbl k)
	{ return Capacitor(c.Capacity / k); }

    /// overloaded operator for output into streams
    friend std::ostream& operator << (std::ostream &out, const Capacitor &c);

  protected:
    virtual void calcVal(Dbl f);

  private:
    Dbl frequency;	// just for caching
};


END_NAMESPACE

#endif // ETECH_CAPACITOR_H


