//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Power.h,v $
// $Id: Power.h,v 1.6 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_POWER_H	// prevent multiple includes
#define ETECH_POWER_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/PhysUnit.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an electrical power
/**
 {\tt Power} represents an electrical power which can be used both in
 DC (direct current) and in AC (alternate current) applications.
 */
class Power: public PhysUnitCplx
{
  public:
    /// constructor
    Power(Dbl r = 0.0, Dbl i = 0.0) : PhysUnitCplx("W",r,i) {}
    /// constructor
    Power(const Cplx &val) : PhysUnitCplx("W",val) {}
    /// copy constructor
    Power(const Power& i) : PhysUnitCplx("W",i.Value) {}

    /// overloaded operator + for addition of two powers
    friend Power operator + (const Power &p1, const Power &p2)
	{ return Power(p1.Value + p2.Value); }
    /// overloaded operator - for subtraction of two powers
    friend Power operator - (const Power &p1, const Power &p2)
	{ return Power(p1.Value - p2.Value); }

    /// overloaded operator * for multiplication of {\tt Power} * {\tt Cplx}
    friend Power operator * (const Power &p1, const Cplx &k)
	{ return Power(p1.Value * k); }
    /// overloaded operator * for multiplication of {\tt Cplx} * {\tt Power}
    friend Power operator * (const Cplx &k, const Power &p1)
	{ return Power(k * p1.Value); }

    /// overloaded operator / for division of {\tt Power} / {\tt Power}
    friend Cplx operator / (const Power &p1, const Power &p2)
	{ return (p1.Value / p2.Value); }
    /// overloaded operator / for division of {\tt Power} / {\tt Cplx}
    friend Power operator / (const Power &p1, const Cplx &k)
	{ return Power(p1.Value / k); }

    /// overloaded operator == for comparison of two Powers
    friend bool operator == (const Power &p1, const Power& p2)
	{ return (p1.Value == p2.Value); }
    /// overloaded operator != for comparison of two Powers
    friend bool operator != (const Power &p1, const Power& p2)
	{ return (p1.Value != p2.Value); }
    /// overloaded operator < for comparison of two Powers
    friend bool operator < (const Power &p1, const Power& p2)
	{ return (p1.Value < p2.Value); }
    /// overloaded operator > for comparison of two Powers
    friend bool operator > (const Power &p1, const Power& p2)
	{ return (p1.Value > p2.Value); }

}; // class Power


END_NAMESPACE

#endif // ETECH_POWER_H
