//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Riff/Chunk.h,v $
// $Id: Chunk.h,v 1.6 2004/04/20 22:25:30 ah Exp $
//
// Description: 
//
//---------------------------------------------------------------------------

#ifndef AH_IO_RIFF_CHUNK_H		// prevent multiple includes
#define AH_IO_RIFF_CHUNK_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Config/Types.h>
#include <AH/Error/Sys.h>
USING_NAMESPACE(AH)

#include <cerrno>
#include <string>
#include <iostream>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/// class representing a RIFF chunk
class IORiffChunk
{
 public:
    /// size of chunk in bytes
#ifdef NO_ANSI_CPLUSPLUS
    enum {size = 8};
#else
    static const uint32 size = 8;
#endif
    /// constructor: create IORiffChunk using specified buffer 
    IORiffChunk(const uint8* chunkbuf);
    /// constructor: create IORiffChunk with chunk on heap
    IORiffChunk
    (
	/// chunk identifier
	const char* id,
	/// chunk data
	uint8* data,
	/// size of chunk data
	uint32 datasize
    );
    /// destructor
    virtual ~IORiffChunk();

    /// get pointer to next chunk:
    virtual const uint8* nextChunk() const
	{ return (chunk + 4 + remSize); }

    /// get pointer to chunk buffer:
    virtual const uint8* chunkBuffer() const
	{ return (chunk); }

    /// get pointer to chunk data:
    virtual const uint8* chunkData() const
	{ return (chunk + 8); }

    // access functions:
    /// return ID of chunk
    const char* getID() const { return id; }
    /// return size of chunk data
    uint32 getDataSize() const { return remSize; }
    /// return size of complete chunk
    uint32 getChunkSize() const { return size + remSize; }

    /// print information about this chunk
    ostream& printInfo(ostream& out, const char* pre = 0);

 protected:
    /// get write pointer to chunk data buffer:
    virtual uint8* chunkDataBuf()
	{ return (heapchunk + 8); }

    uint8*		heapchunk;	// chunk on heap
    const uint8*	chunk;		// pointer to start of chunk
 private:
    char		id[5];		// chunk identifier
    uint32		remSize;	// remaining chunk length
    bool		onHeap;		// flag if chunk is on heap
};

END_NAMESPACE

#endif // !AH_IO_RIFF_CHUNK_H
