//---------------------------------------------------------------------------
// Copyright 2005 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Test/Log.h,v $
// $Id: Log.h,v 1.2 2005/09/25 23:44:37 ah Exp $
//
// Description: Test Logger Utility
//
//---------------------------------------------------------------------------

#ifndef AH_TEST_LOG_H		// prevent multiple includes
#define AH_TEST_LOG_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
USING_NAMESPACE(AH)

#include <string>
#include <iostream>

USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/** test logger utility. */
class TestLog
{
 public:
    /** constructor. */
    TestLog(ostream& o, const std::string& name, bool v) 
	: errNum(0), out(o), verbose(v), reported(false), testName(name), testNum(0)
    {
	out << "Test of " << testName << ":" << std::endl;
    }

    /** announce a new test series. */
    void announceSeries(const std::string& msg)
    {
	out << "Test of " << msg << ":" << std::endl;
    }

    /** announce a new test. */
    void announceTest(const std::string& msg = "")
    {
	out << "Test " << ++testNum << ": ";
	if (verbose && msg.length()) out << msg << ": ";
	out.flush();
    }

    /** print returned value. */
    template <class T> void returned(const T& value)
    {
	if (verbose)
	{
	    out << "returns \"" << value << "\": ";
	    out.flush();
	}
    }

    /** print failure message, with expected value. */
    template <class T> void failed(const T& expected)
    {
	++errNum;
	if (verbose)
	{
	    out << ", expected=\"" << expected << "\": ";
	}
	out << "failed." << std::endl;
    }

    /** print passed message. */
    void passed()
    {
	out << "passed." << std::endl;
    }

    /** report results. */
    int makeReport()
    {
	if (errNum)
	    out << "Warning: "
		<< errNum << " error" << (errNum > 1 ? "s" : "") 
		<< " detected in "
		<< testNum << " test" << (testNum > 1 ? "s" : "") 
		<< std::endl;
	else
	    out << "Okay: no errors detected in "
		<< testNum << " test" << (testNum > 1 ? "s" : "") 
		<< std::endl;
	reported = true;
	if (errNum) return 1;
	return 0;
    }

    /** return number of detected errors. */
    unsigned int getErrNum() const
	{ return errNum; }

    /** destructor. */
    ~TestLog()
    {
	if (!reported) makeReport();
    }

 protected:
    unsigned int errNum;

    bool isVerbose() const { return verbose; }

 private:
    ostream& out;
    bool verbose;
    bool reported;
    std::string testName;
    unsigned int testNum;
};

END_NAMESPACE

#endif // !AH_TEST_LOG_H

