//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/testalgo.c,v $
// $Id: testalgo.c,v 1.6 2004/04/20 22:14:57 ah Exp $
//
// Description: test program for testing the algorithms in <Etech/Algo.h>
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: testalgo.c,v 1.6 2004/04/20 22:14:57 ah Exp $ (c) 1997 Andreas Huennebeck";

// --- necessary includes ---

#include <cstdlib>
#include <iostream>
#include <cmath>

#include <AH/Etech/Etech.h>

USING_NAMESPACE(Ah)

// --- global function implementation ---

int main()
{
    std::cout << "Impedance Testprogram" << std::endl;

    //---------------------------------------------------------------------
    std::cout << "testing algorithms using Current, Voltage, Impedance, Power:" 
	 << std::endl;
    Current i(.25);
    Voltage u(1.5);

    Impedance z = u / i;
    Power p = u * i;

    std::cout << "  i=" << i << ", u=" << u << ":\n"
	 << "  z=" << z << " und p=" << p << std::endl;

    //---------------------------------------------------------------------
    std::cout << "testing Class Impedance:" << std::endl;
    Impedance z1(10,0);
    Impedance z2(0,5);

    std::cout.precision(4); std::cout.setf(std::ios::fixed, std::ios::floatfield);
    std::cout << "  z1="; std::cout.width(10); 
    std::cout << z1 << " / " << abs(z1) << " (" << inVdB(abs(z1)) << "VdB), phi=" 
         << inDeg(arg(z1)) << std::endl;
    std::cout << "  z2="; std::cout.width(10);
    std::cout << z2 << " / " << abs(z2) << " (" << inVdB(abs(z2)) << "VdB), phi=" 
       << inDeg(arg(z2)) << std::endl;

    Impedance z3 = z1 + z2;
    std::cout << "  z3= z1 + z2  =" << z3 << std::endl;
    z3 = z1 + z2;
    std::cout << "  z3= z1 + z2  =" << z3 << std::endl;
    z3 = z1 || z2;
    std::cout << "  z3= z1 || z2 =" << z3 << std::endl;

    //---------------------------------------------------------------------
    std::cout << "testing Class Resistor:" << std::endl;
    Resistor r1(10);
    Resistor r2(5);

    std::cout << "  r1=" << r1 << std::endl;
    std::cout << "  r2=" << r2 << std::endl;

    Resistor r3 = r1 + r2;
    std::cout << "  r3= r1 + r2  =" << r3 << std::endl;
    r3 = r1 || r2;
    std::cout << "  r3= r1 || r2 =" << r3 << std::endl;

    //---------------------------------------------------------------------
    std::cout << "testing class Capacitor:" << std::endl;

    Dbl frequency = 1000.0;    // Hz
    Capacitor c1(1e-6);
    Capacitor c2(5e-7);

    std::cout << "  c1=" << c1 << std::endl;
    std::cout << "  c2=" << c2 << std::endl;

    Capacitor c3 = c1 + c2;
    std::cout << "  c3= c1 + c2  =" << c3 << std::endl;
    c3 = c1 || c2;
    std::cout << "  c3= c1 || c2 =" << c3 << std::endl;

    z = c3(frequency);
    std::cout << "  z = c1 || c2 =" << z << std::endl;

    //---------------------------------------------------------------------
    std::cout << "testing class Inductor:" << std::endl;
    Inductor l1(10);
    Inductor l2(5);

    std::cout << "  l1=" << l1 << std::endl;  
    std::cout << "  l2=" << l2 << std::endl;

    Inductor l3 = l1 + l2;
    std::cout << "  l3= l1 + l2  =" << l3 << std::endl;
    l3 = l1 || l2;
    std::cout << "  l3= l1 || l2 =" << l3 << std::endl;

    z = l3(frequency);
    std::cout << "  z = l1 || l2 =" << z << std::endl;

    //---------------------------------------------------------------------
    std::cout << "Test finished." << std::endl;

    return (0);
}
