//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/AudioFile.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: AudioFile.c,v 1.2 2005/11/03 21:52:00 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/AudioFile.h>
#include <AH/Error/Intern.h>
#include <AH/Error/Sys.h>
#include <AH/Error/Basic.h>
USING_NAMESPACE(AH)

#include <sstream>
#include <cstdio>
#include <string>
USING_NAMESPACE(std)

BEGIN_NAMESPACE(AH)




// return audio file information from SF_INFO struct as string
std::string IOAudioFileInfo::getSfInfo(const SF_INFO& info)
{
    std::ostringstream oss;
    oss << "  " << info.frames << " samples with " << info.channels 
	<< " channel" << (info.channels > 1 ? "s":"") 
        << " at " << info.samplerate << " samples/sec in " 
	<< info.sections << " section" << (info.sections > 1 ? "s":"") 
	<< std::endl;
    oss << "  format: " << IOAudioFileInfo::getSfInfoFormat(info.format) 
	<< std::endl;

    oss << "  bits/sample: ";
    int bits = IOAudioFileInfo::getSampleBits(info);
    if (bits < 0)      oss << -bits << " bits floating point" << std::endl;
    else if (bits > 0) oss << bits << " bits fixed point" << std::endl;
    else               oss << " unknown" << std::endl;

    oss << "  seekable = " << info.seekable << std::endl;
    return oss.str();
}



// return format information from SF_INFO struct member 'format' as string
std::string IOAudioFileInfo::getSfInfoFormat(int format)
{
    class Local // used as dummy to be able to define local functions
    {
    public:
	static std::string type2string(int type)
	{
	    switch (type)
	    {
	    case SF_FORMAT_WAV:		return "Microsoft WAV format (little endian)";
	    case SF_FORMAT_AIFF:	return "Apple/SGI AIFF format (big endian)";
	    case SF_FORMAT_AU:		return "Sun/NeXT AU format (big endian)";
	    case SF_FORMAT_RAW:		return "RAW PCM data";
	    case SF_FORMAT_PAF:		return "Ensoniq PARIS file format";
	    case SF_FORMAT_SVX:		return "Amiga IFF / SVX8 / SV16 format";
	    case SF_FORMAT_NIST:	return "Sphere NIST format";
	    case SF_FORMAT_VOC:		return "VOC files";
	    case SF_FORMAT_IRCAM:	return "Berkeley/IRCAM/CARL";
	    case SF_FORMAT_W64:		return "Sonic Foundry's 64 bit RIFF/WAV";
	    case SF_FORMAT_MAT4:	return "Matlab (tm) V4.2 / GNU Octave 2.0";
	    case SF_FORMAT_MAT5:	return "Matlab (tm) V5.0 / GNU Octave 2.1";
	    default:			return "unknown type";
	    }
	}

	static std::string subtype2string(int subtype)
	{
	    switch (subtype)
	    {
	    case SF_FORMAT_PCM_S8:	return "Signed 8 bit data";
	    case SF_FORMAT_PCM_16:	return "Signed 16 bit data";
	    case SF_FORMAT_PCM_24:	return "Signed 24 bit data";
	    case SF_FORMAT_PCM_32:	return "Signed 32 bit data";

	    case SF_FORMAT_PCM_U8:	return "Unsigned 8 bit data (WAV and RAW only)";

	    case SF_FORMAT_FLOAT:	return "32 bit float data";
	    case SF_FORMAT_DOUBLE:	return "64 bit float data";

	    case SF_FORMAT_ULAW:	return "U-Law encoded";
	    case SF_FORMAT_ALAW:	return "A-Law encoded";
	    case SF_FORMAT_IMA_ADPCM:	return "IMA ADPCM";
	    case SF_FORMAT_MS_ADPCM:	return "Microsoft ADPCM";

	    case SF_FORMAT_GSM610:	return "GSM 6.10 encoding";
	    case SF_FORMAT_VOX_ADPCM:	return "Oki Dialogic ADPCM encoding";

	    case SF_FORMAT_G721_32:	return "32kbs G721 ADPCM encoding";
	    case SF_FORMAT_G723_24:	return "24kbs G723 ADPCM encoding";
	    case SF_FORMAT_G723_40:	return "40kbs G723 ADPCM encoding";

	    case SF_FORMAT_DWVW_12:	return "12 bit Delta Width Variable Word encoding";
	    case SF_FORMAT_DWVW_16:	return "16 bit Delta Width Variable Word encoding";
	    case SF_FORMAT_DWVW_24:	return "24 bit Delta Width Variable Word encoding";
	    case SF_FORMAT_DWVW_N:	return "N bit Delta Width Variable Word encoding";
	    default:			return "unknown subtype";
	    }
	}

	static std::string endian2string(int endian)
	{
	    switch (endian)
	    {
	    case SF_ENDIAN_FILE:	return "Default file endian-ness";
	    case SF_ENDIAN_LITTLE:	return "Force little endian-ness";
	    case SF_ENDIAN_BIG:		return "Force big endian-ness";
	    case SF_ENDIAN_CPU:		return "Force CPU endian-ness";
	    default:			return "unknown endianness";
	    }
	}
    };

    return Local::type2string(format & SF_FORMAT_TYPEMASK) + ", " + 
	   Local::subtype2string(format & SF_FORMAT_SUBMASK) + ", " + 
	   Local::endian2string(format & SF_FORMAT_ENDMASK);
}

 
// get minimum number of sample bits of the audio file
int IOAudioFileInfo::getSampleBits(const SF_INFO& info)
{
    switch (info.format & SF_FORMAT_SUBMASK)
    {
    case SF_FORMAT_PCM_S8:	return  8;   // Signed 8 bit data
    case SF_FORMAT_PCM_16:	return  16;  // Signed 16 bit data
    case SF_FORMAT_PCM_24:	return  24;  // Signed 24 bit data
    case SF_FORMAT_PCM_32:	return  32;  // Signed 32 bit data

    case SF_FORMAT_PCM_U8:	return   8;  // Unsigned 8 bit data (WAV and RAW only)

    case SF_FORMAT_FLOAT:	return -32;  // 32 bit float data
    case SF_FORMAT_DOUBLE:	return -64;  // 64 bit float data

    case SF_FORMAT_ULAW:	return   0;  // U-Law encoded
    case SF_FORMAT_ALAW:	return   0;  // A-Law encoded
    case SF_FORMAT_IMA_ADPCM:	return   0;  // IMA ADPCM
    case SF_FORMAT_MS_ADPCM:	return   0;  // Microsoft ADPCM

    case SF_FORMAT_GSM610:	return   0;  // GSM 6.10 encoding
    case SF_FORMAT_VOX_ADPCM:	return   0;  // Oki Dialogic ADPCM encoding

    case SF_FORMAT_G721_32:	return   0;  // 32kbs G721 ADPCM encoding
    case SF_FORMAT_G723_24:	return   0;  // 24kbs G723 ADPCM encoding
    case SF_FORMAT_G723_40:	return   0;  // 40kbs G723 ADPCM encoding

    case SF_FORMAT_DWVW_12:	return  12;  // 12 bit Delta Width Variable Word encoding
    case SF_FORMAT_DWVW_16:	return  16;  // 16 bit Delta Width Variable Word encoding
    case SF_FORMAT_DWVW_24:	return  24;  // 24 bit Delta Width Variable Word encoding
    case SF_FORMAT_DWVW_N:	return   0;  // N bit Delta Width Variable Word encoding
    default:			return   0;  // unknown subtype
    }
}

END_NAMESPACE
