//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU Library General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING.LIB' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/CFormat.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: CFormat.c,v 1.5 2004/12/31 16:04:34 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/IO/CFormat.h>
USING_NAMESPACE(AH)

#include <iostream>
USING_NAMESPACE(std)

// --- local stuff ---

#define TRACE 0

// --- class member implementation ---

BEGIN_NAMESPACE(AH)


// ----------------------------------------------------------------------
// constructor:
// ----------------------------------------------------------------------
IOCFormat::IOCFormat(const string& format)
    : fill(' '), precision(0), width(0), 
      base	((ios::fmtflags)0), // it is save to cast to int here as
      floating	((ios::fmtflags)0), // the flags are not a real enum but
      adjust	((ios::fmtflags)0), // a bitmap covering parallel bits, e.g.
      showpos	((ios::fmtflags)0), // 0x01 | 0x04
      showbase(false), setupper(false), setprecision(false), 
      hash(false), error(false)
{
    init(format.c_str());
} // end of constructor -------------------------------------------------



// ----------------------------------------------------------------------
// constructor:
// ----------------------------------------------------------------------
IOCFormat::IOCFormat(const char* fmt)
    : fill(' '), precision(0), width(0), 
      base	((ios::fmtflags)0), // it is save to cast to int here as
      floating	((ios::fmtflags)0), // the flags are not a real enum but
      adjust	((ios::fmtflags)0), // a bitmap covering parallel bits, e.g.
      showpos	((ios::fmtflags)0), // 0x01 | 0x04
      showbase(false), setupper(false), setprecision(false),
      hash(false), error(false)
{
    init(fmt);
} // end of constructor -------------------------------------------------



// ----------------------------------------------------------------------
// scan format string and save info about how to set the ostream later
// ----------------------------------------------------------------------
void IOCFormat::init(const char* format)
{
    //----------------------
    // handle bad argument:
    //----------------------
    if (!format) 
    {
	error = true;
	return;
    }
#if TRACE
    cout << "format=\"" << format << "\"" << endl;
#endif

    //-------------------------------------------------
    // handle illegal argument and missing parameters:
    //-------------------------------------------------
    unsigned int len = strlen(format);
    if ((2 > len) || ('%' != format[0]))
    {
	error = true;
	return;
    }
    unsigned int pos = 1;	// start with 2. character

    //------------------------------------
    // handle all available format flags:
    //------------------------------------
#if TRACE
    cout << "handle flags(\"" << &format[pos] << "\")" << endl;
#endif
    while (pos < len)
    {
	bool finished = false;
	switch (format[pos])
	{
	case '0': fill = '0';			break;
	case '#': hash = true;			break;
	case '-': adjust = ios::left;		break;
	case '+': showpos = ios::showpos;	break;
	default : finished = true;		break;
	}
	if (finished) break;
	pos++;
    }

    //----------------------------
    // handle format field width:
    //----------------------------
    if (isdigit(format[pos]))
    {
#if TRACE
	cout << "handle width(\"" << &format[pos] << "\")" << endl;
#endif
	// read width and advance to next argument: 
	width = atoi(&format[pos]);
	while ((pos < len) && isdigit(format[pos]))
	    pos++;
    }

    //--------------------------
    // handle format precision:
    //--------------------------
    if ((pos < len) && ('.' == format[pos]))
    {
#if TRACE
	cout << "handle precision(\"" << &format[pos] << "\")" << endl;
#endif
	pos++;
	if ((pos < len) && isdigit(format[pos]))
	{
	    // read precision and advance to next argument: 
	    setprecision = true;
	    precision = atoi(&format[pos]);
	    while ((pos < len) && isdigit(format[pos]))
		pos++;
	}
    }
    
    //----------------------------
    // handle parameter size 'l':
    //----------------------------
    if (pos < len)
    {
#if TRACE
	cout << "handle parameter size(\"" << &format[pos] << "\")" << endl;
#endif
	switch (format[pos])
	{
	case 'h': pos++; break;	// ignore
	case 'L': pos++; break;	// ignore
	case 'l': pos++;	// ignore "l"
		  if ('l' == format[pos])
		      pos++;	// ignore "ll"
		  break;
	default:  break;
	}
    }

    //----------------------------
    // handle type of conversion:
    //----------------------------
    if (pos < len)
    {
#if TRACE
	cout << "handle conversion(\"" << &format[pos] << "\")" << endl;
#endif
	switch (format[pos])
	{
	//---------
	// decimal
	//---------
	case 'd':
	    floating = ios::fixed; 
	    base = ios::dec;
	    if (('0' == fill) && (ios::left != adjust))
		adjust = ios::internal;
	    break;

	//-------------
	// hexadecimal
	//-------------
	case 'X': 
	    setupper = true;
	    // fall through
	case 'x': 
	    base = ios::hex;
	    if (hash) showbase = true; 
	    break;

	//-------
	// octal
	//-------
	case 'o': 
	    base = ios::oct; 
	    if (hash) showbase = true; 
	    break;

	//-------
	// float
	//-------
	case 'f': 
	    if (hash) showpos = ios::showpos;
	    floating = ios::fixed; 
	    break;

	case 'E':
	    setupper = true;
	    // fall through
	case 'e': 
	    if (hash) showpos = ios::showpos;
	    floating = ios::scientific;
	    if (!setprecision)
	    {
		precision = 6;
		setprecision =true;
	    }
	    break;

	//--------
	// string
	//--------
	case 's':
	    if ('0' == fill) fill = ' ';
	    break;
	default: break; // ignore
	}
    }
#if TRACE
    cerr << "precision=" << precision << ", "
	 << "adjust=" << adjust << ", "
	 << "floating=" << floating << "\n"
	 << "base=" << base << ", "
	 << "showpos=" << showpos << ", "
	 << "setupper=" << setupper << "\n"
	 << "setprecision=" << setprecision << "\n"
	 << "showbase=" << showbase << ", "
	 << "fill='" << fill << "', "
	 << "width=" << width << endl;
#endif
} // end of init() ------------------------------------------------------



// ----------------------------------------------------------------------
// manipulator function: manip()
// set ostream flags according to info
// ----------------------------------------------------------------------
ostream& IOCFormat::manip(ostream& out) const
{
#if 1
    if (!error)
    {
	// set ostream:
	if (setprecision) out.precision(precision);
	if (adjust)    out.setf(adjust, ios::adjustfield);
	if (floating)  out.setf(floating, ios::floatfield);
	if (base)      out.setf(base, ios::basefield);
	if (showpos)   out.setf(showpos);
	if (setupper)  out.setf(ios::uppercase);
	else	       out.unsetf(ios::uppercase);
	if (showbase)  out.setf(ios::showbase);
	else	       out.unsetf(ios::showbase);
	out.fill(fill);
	if (width)     out.width(width);
    }
#else
    if (!error)
    {
	//--------------
	// set ostream:
	//--------------
	out.precision(precision);
	if (adjust)    out.setf(adjust, ios::adjustfield);
	if (floating)  out.setf(floating, ios::floatfield);
#if 1
	else if (width != out.width()) out.width(width);
//	else if (width) out.width(width);
#else
	if (width)     out.width(width);
#endif
	if (base)      out.setf(base, ios::basefield);
	if (showpos)   out.setf(showpos);
	if (setupper)  out.setf(ios::uppercase);
	else	       out.unsetf(ios::uppercase);
	if (showbase)  out.setf(ios::showbase);
	else	       out.unsetf(ios::showbase);
#if 1
	if (fill != out.fill())
#endif
	out.fill(fill);
    }
#endif
    return out;
} // end of manip()------------------------------------------------------

END_NAMESPACE
