#include <AH/CUtil/SecureArray.h>
#include <AH/Test/UnitTest.h>

using namespace AH;

//-----------------
// our test class:
//-----------------
class Test: public UnitTest
{
    // test functions, to be called in Test::run():
    void demo1();
    void demo2();
    void demo3();
    void demo4();

    // run all tests:
    virtual void run(const std::vector<std::string>&)
    {
	this->demo1();
	this->demo2();
	this->demo3();
	this->demo4();
    }

 public:
    // constructor
    Test(): UnitTest("Demo", std::cout) {}

}; 

UnitTest* theUnitTest =  new Test();


void Test::demo1()
{ 
    this->announceTestSeries("demo1");

    // test data (without bug):
    const unsigned int data[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

    // run the tests:
    const unsigned int testnum = CUtilSecureArray::getNumEntries(data);
    for (unsigned int i = 0; i < testnum; ++i)
    {
	UNIT_TEST_BEGIN
	this->checkEqual("data[i]", data[i], i);
        UNIT_TEST_END
    }
} // end of Test::demo1()


void Test::demo2()
{ 
    this->announceTestSeries("demo2");

    // test data (with bug):
    const unsigned int data[] = { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10 };

    // run the tests:
    const unsigned int testnum = CUtilSecureArray::getNumEntries(data);
    for (unsigned int i = 0; i < testnum; ++i)
    {
	UNIT_TEST_BEGIN
	this->checkEqual("data[i]", data[i], i);
        UNIT_TEST_END
    }
} // end of Test::demo2()


void Test::demo3()
{ 
    this->announceTestSeries("demo3");

    // test data (with bug and exception):
    const double data[] = { 0, 1, 2, 3, 4, 5, 6, 8, 9, 10 };
    class myException: public std::exception
    {
	std::string errmsg;
    public:
	myException(const std::string& msg) : errmsg(msg) {}
	virtual ~myException() throw() {}
	const char* what() const throw() { return errmsg.c_str(); }
    };

    // run the tests:
    const unsigned int testnum = CUtilSecureArray::getNumEntries(data);
    for (unsigned int i = 0; i < testnum; ++i)
    {
	UNIT_TEST_BEGIN
	double x = sqrt(data[i]); // the function that could throw an exception
	if (i == 7) throw "hallo";
	if (i == 8) throw myException("my exception");

	this->checkEqual("sqrt()", x+ (0.002 * i), x, 0.01);
        UNIT_TEST_END
    }
} // end of Test::demo3()


void Test::demo4()
{ 
    this->announceTestSeries("demo4");

    class myException: public std::exception
    {
	std::string errmsg;
    public:
	myException(const std::string& msg) : errmsg(msg) {}
	virtual ~myException() throw() {}
	const char* what() const throw() { return errmsg.c_str(); }
    };

    // run the tests:
    for (unsigned int i = 0; i < 2; ++i)
    {
	UNIT_TEST_EXCEPTION_BEGIN(myException)
	if (i == 1) throw myException("my exception");
	UNIT_TEST_EXCEPTION_END(myException)
    }
} // end of Test::demo4()


