//---------------------------------------------------------------------------
// Copyright 2005 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// Additionally this source is licensed exclusively to the company
// Bruker Analytik GmbH under the terms of the GNU Lesser General Public 
// License (LGPL). See the file 'COPYING.LIB' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHtest/UnitTest.c,v $
// $Id: UnitTest.c,v 1.3 2005/09/28 05:57:23 ah Exp $
//
// Description: Unit Test Utility
//
//---------------------------------------------------------------------------

static const char cvsid[] = "$Id: UnitTest.c,v 1.3 2005/09/28 05:57:23 ah Exp $ (c) 2005 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Test/UnitTest.h>
USING_NAMESPACE(AH)

BEGIN_NAMESPACE(AH)


/** constructor. */
UnitTest::UnitTest(const std::string& name, std::ostream& Out)
    : out(Out), testName(name), testNum(0), failedNum(0),
      verbose(false), quiet(false)
{
    this->out << "****** Test of " << this->testName << " ******" << std::endl; 
} // constructor


/** destructor. */
UnitTest::~UnitTest()
{
    this->out << this->testName << ": "; 
    if (this->failedNum)
	this->out << this->failedNum << " error" << (this->failedNum != 1 ? "s" : "");
    else
	this->out << "no errors"; 
    this->out << " found in " << this->testNum << " test"
	      << (this->testNum != 1 ? "s. " : ". ");
    if (this->failedNum)
	this->out << "failed.";
    else
	this->out << "passed.";
    this->out << std::endl;
}; // destructor


/** show help. */
void UnitTest::printHelp(const char* progName)
{
    this->out << "usage: " << progName << " [-vqdh" 
	      << this->addOptions << "] " 
	      << this->addArguments << std::endl;
    this->out << " -v       verbose mode" <<  std::endl;
    this->out << " -q       quiet mode" <<  std::endl;
    this->out << " -d       increase debug level" <<  std::endl;
    this->out << " -h       show this help message" <<  std::endl;
    this->printUserHelp(); // print additional user spcecified options
    exit(1); 
}; // destructor


END_NAMESPACE

/* main function */
int main(int argc, const char* argv[])
{
    // sanity check:
    if (!theUnitTest)
    {
	std::cerr << "Error: 'theUnitTest' is not created" << std::endl;
	exit(1);
    }

    // set options:
    std::vector<std::string> arguments;
    int debuglevel = 0;
    for (int i = 1; i < argc; ++i)
    {
	if ('-' == *argv[i])
	{
	    // its an option:
	    bool found = false;
	    for (const char* cp = argv[i] + 1; *cp; ++cp)
	    {
		switch (*cp)
		{
		case 'v': theUnitTest->setVerbose(true); found = true; break;
		case 'q': theUnitTest->setQuiet(true); found = true; break;
		case 'd': ++debuglevel; found = true; break;
		case 'h': theUnitTest->printHelp(argv[0]); break;
		default: break;
		}
	    }
	    if (!found)
		arguments.push_back(argv[i]); // catch all other arguments
	}
	else
	    arguments.push_back(argv[i]); // catch all other arguments
    }
    theUnitTest->setDebugLevel(debuglevel);


    // run the tests:
    theUnitTest->run(arguments);

    // determine the exit value and finish:
    int exitval = theUnitTest->testsFailed() ? 1 : 0;
    delete theUnitTest;
    exit(exitval);
} // main()
