//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/datacopy.h,v $
// $Id: datacopy.h,v 1.3 2006/04/06 21:16:28 ah Exp $ 
//
// Description: cut a pice out of a mp3 file into a new mp3 file
//
//---------------------------------------------------------------------------

#ifndef AH_WAVTOOL_DATACOPY_H
#define AH_WAVTOOL_DATACOPY_H

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <string>
USING_NAMESPACE(std)

#include <AH/IO/Riff/WaveFile.h>
USING_NAMESPACE(AH)

// --- classes ---

class DataCopyRaw
{
 public:
    /// constructor
    DataCopyRaw(IORiffWaveFile& inputStream, const string& inputName,
		IORiffWaveFile& outputStream, const string& outputName,
		int v = 0)
	: verbose(v), in(inputStream), inputFileName(inputName),
	  out(outputStream), outputFileName(outputName) {}

    /// destructor
    virtual ~DataCopyRaw() {}

    /// copy specified data range
    void copy(int s1, int s2);

 protected:

    /// read data buffer
    int readBuf(unsigned char* buf, unsigned int size);

    /// process data buffer
    virtual void processBuf
    (
	uint32 pos,
	uint32 max,
	unsigned char* buf,
	unsigned int size
    ) {}

    /// write data buffer
    int writeBuf(const unsigned char* buf, unsigned int size);

    int verbose;

 private:
    IORiffWaveFile& in;
    const string& inputFileName;
    IORiffWaveFile& out;
    const string& outputFileName;

}; // class DataCopyRaw


class DataCopyFade: public DataCopyRaw
{
 public:
    /// constructor
    DataCopyFade(IORiffWaveFile& inputStream, const string& inputName,
		 IORiffWaveFile& outputStream, const string& outputName,
		 int v = 0)
	: DataCopyRaw(inputStream, inputName, outputStream, outputName, v)
	{}

    /// process data buffer (fade in)
    virtual void processBuf
    (
	uint32 pos,
	uint32 max,
	unsigned char* buf,
	unsigned int size
    );

 protected:
    /// get fade factor
    virtual double getFadeFactor
    (
	uint32 pos, 
	uint32 max, 
	double rate, 
	uint32 sw_pos, 
	double m
    ) = 0;

#if 1 // don't know why gcc3.4.3 does not like static const 'type':
    /// theoretical maximal damping done in exponential mode
#   define MAX_DB 40.0
    /// factor where we start to do linear damping
#   define SWITCH 0.9
    /// damping value below which we do linear damping
#   define SW_DB (MAX_DB * SWITCH)
#else
    /// theoretical maximal damping done in exponential mode
    static const double MAX_DB = 40;
    /// factor where we start to do linear damping
    static const double SWITCH = 0.9;
    /// damping value below which we do linear damping
    static const double SW_DB = MAX_DB * SWITCH;
#endif
}; // class DataCopyFade


class DataCopyFadeIn: public DataCopyFade
{
 public:
    /// constructor
    DataCopyFadeIn(IORiffWaveFile& inputStream, const string& inputName,
		   IORiffWaveFile& outputStream, const string& outputName,
		   int v = 0)
	: DataCopyFade(inputStream, inputName, outputStream, outputName, v)
	{}

 protected:
    /// get fade in factor
    virtual double getFadeFactor
    (
	uint32 pos, 
	uint32 max, 
	double rate, 
	uint32 sw_pos, 
	double m
    );

}; // class DataCopyFadeIn



class DataCopyFadeOut: public DataCopyFade
{
 public:
    /// constructor
    DataCopyFadeOut(IORiffWaveFile& inputStream, const string& inputName,
		   IORiffWaveFile& outputStream, const string& outputName,
		   int v = 0)
	: DataCopyFade(inputStream, inputName, outputStream, outputName, v)
	{}

 protected:
    /// get fade out factor
    virtual double getFadeFactor
    (
	uint32 pos, 
	uint32 max, 
	double rate, 
	uint32 sw_pos, 
	double m
    );

}; // class DataCopyFadeOut


#endif // AH_WAVTOOL_DATACOPY_H
// --- end of file ---


