//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/fileset.c,v $
// $Id: fileset.c,v 1.6 2004/04/20 22:43:37 ah Exp $ 
//
// Description: cut a pice out of a mp3 file into a new mp3 file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: fileset.c,v 1.6 2004/04/20 22:43:37 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <cstdlib>
#ifndef NO_UNISTD
#   include <unistd.h>
#endif
//#include <string.h>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
USING_NAMESPACE(std)

#include <AH/Error/Class.h>
#include <AH/Error/Basic.h>
#include <AH/STL/String.h>
#include "fileset.h"

USING_NAMESPACE(AH)

// --- local constants ---

// --- local variables ---

// --- local classes ---

// add new file name
void FileSettings::addFileName(const string& name)
{
    fileName.push_back(name);
    if (sampleRange.size() < fileName.size()) 
	sampleRange.push_back("");
    if (timeRange.size() < fileName.size()) 
	timeRange.push_back("");
    if (fadeIn.size() < fileName.size()) 
	fadeIn.push_back("");
    if (fadeOut.size() < fileName.size()) 
	fadeOut.push_back("");
    if (silence.size() < fileName.size()) 
	silence.push_back(-1);
}


// add sample settings
void FileSettings::addSampleRange(const string& sr)
{
    if (sampleRange.size() > fileName.size())
	sampleRange[sampleRange.size() - 1] = sr;
    else
	sampleRange.push_back(sr);
}


// add time settings
void FileSettings::addTimeRange(const string& sr)
{
    if (timeRange.size() > fileName.size())
	timeRange[timeRange.size() - 1] = sr;
    else
	timeRange.push_back(sr);
}


// add fade in settings
void FileSettings::addFadeIn(const string& sr)
{
    if (fadeIn.size() > fileName.size())
	fadeIn[fadeIn.size() - 1] = sr;
    else
	fadeIn.push_back(sr);
}


// add fade out settings
void FileSettings::addFadeOut(const string& sr)
{
    if (fadeOut.size() > fileName.size())
	fadeOut[fadeOut.size() - 1] = sr;
    else
	fadeOut.push_back(sr);
}


// add silence settings
void FileSettings::addSilence(const string& si)
{
    string siltime = si;
    STLStringDelSurround(siltime);

    double val;
    if (siltime == "") val = -1;
    else val = atof(siltime.c_str());

    silence.push_back(val);
    addFileName("silence");
}


// get number of entries
unsigned int FileSettings::getNum() const 
{
    return fileName.size(); 
}


// get specified filename
const string& FileSettings::getFileName(unsigned int i) const
{
    if (i < fileName.size()) return fileName[i]; 
    return empty;
}


// get specified sample range
const string& FileSettings::getSampleRange(unsigned int i) const
{
    if (i < sampleRange.size()) return sampleRange[i]; 
    return empty;
}

// get specified sample range
void FileSettings::getSampleRange(unsigned int i, int& f1, int& f2) const
{
    f1 = f2 = -1; // default

    string range = getSampleRange(i);
    STLStringDelSurround(range);
    if (range == "") return;
    
    vector<string> token;
    STLStringToken(range, token, "-");
    switch (token.size())
    {
    default:
    case 0: return;
    case 1:
	if ('-' == range[0]) f2 = atoi(token[0].c_str());
	else		     f1 = atoi(token[0].c_str());
	return;
    case 2: 
	f1 = atoi(token[0].c_str());
	f2 = atoi(token[1].c_str());
	return;
    }
}


// get specified fade in time
double FileSettings::getFadeInTime(unsigned int i) const
{
    if (i >= fadeIn.size()) return 0;

    string fade = fadeIn[i];
    STLStringDelSurround(fade);
    if (fade == "") return 0;
    
    double val = atof(fade.c_str());
    if (val < 0) return 0;
    return val;
}


// get specified fade out time
double FileSettings::getFadeOutTime(unsigned int i) const
{
    if (i >= fadeOut.size()) return 0;

    string fade = fadeOut[i];
    STLStringDelSurround(fade);
    if (fade == "") return 0;
    
    double val = atof(fade.c_str());
    if (val < 0) return 0;
    return val;
}


// get specified silence
double FileSettings::getSilenceTime(unsigned int i) const
{
    if (i < silence.size()) return silence[i]; 
    return 0;
}


// get specified time range
const string& FileSettings::getTimeRange(unsigned int i) const
{
    if (i < timeRange.size()) return timeRange[i]; 
    return empty;
}


static double convStr2Time(const string& s)
{
    vector<string> token;
    STLStringToken(s, token, ":");
    switch (token.size())
    {
    default:
    case 0: return -1;
    case 1: return atof(s.c_str());
    case 2: 
        {
	    int min = atoi(token[0].c_str());
	    double sec = atof(token[1].c_str());
	    return (double)(60 * min) + sec;
	}
    }
    return -1; // to keep compiler quiet
}


// get specified time range
void FileSettings::getTimeRange(unsigned int i, double& t1, double& t2) const
{
    t1 = t2 = -1; // default

    string range = getTimeRange(i);
    STLStringDelSurround(range);
    if (range == "") return;

    vector<string> token;
    STLStringToken(range, token, "-");
    switch (token.size())
    {
    default:
    case 0: return;
    case 1:
	if ('-' == range[0]) t2 = convStr2Time(token[0].c_str());
	else		     t1 = convStr2Time(token[0].c_str());
	return;
    case 2: 
	t1 = convStr2Time(token[0].c_str());
	t2 = convStr2Time(token[1].c_str());
	return;
    }
}


// --- end of file ---

















