//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/fileset.h,v $
// $Id: fileset.h,v 1.5 2004/04/20 22:43:37 ah Exp $ 
//
// Description: cut a pice out of a mp3 file into a new mp3 file
//
//---------------------------------------------------------------------------

#ifndef AH_MP3TOOL_FILESET_H
#define AH_MP3TOOL_FILESET_H

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <cstdlib>
#ifndef NO_UNISTD
#   include <unistd.h>
#endif
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
USING_NAMESPACE(std)

#include <AH/Error/Class.h>
#include <AH/Error/Basic.h>
USING_NAMESPACE(AH)

// --- classes ---

class FileSettings
{
 public:
    /// add new file name
    void addFileName(const string& name);
    /// add sample settings
    void addSampleRange(const string& sr);
    /// add time settings
    void addTimeRange(const string& tr);
    /// add fade in settings
    void addFadeIn(const string& fi);
    /// add fade out settings
    void addFadeOut(const string& fo);
    /// add silence
    void addSilence(const string& si);

    /// get number of entries
    unsigned int getNum() const;

    /// get specified filename
    const string& getFileName(unsigned int i) const;

    /// get specified sample range
    const string& getSampleRange(unsigned int i) const;
    /// get specified sample range
    void getSampleRange(unsigned int i, int& f1, int& f2) const;

    /// get specified time range
    const string& getTimeRange(unsigned int i) const;
    /// get specified time range
    void getTimeRange(unsigned int i, double& t1, double& t2) const;

    /// get specified fade in time
    double getFadeInTime(unsigned int i) const;
    /// get specified fade out time
    double getFadeOutTime(unsigned int i) const;
    /// get specified silence time
    double getSilenceTime(unsigned int i) const;

 private:
    vector<string> fileName;
    vector<string> sampleRange;
    vector<string> timeRange;
    vector<string> fadeIn;
    vector<string> fadeOut;
    vector<double> silence;
    string empty;

}; // class FileSettings


#endif // AH_MP3TOOL_FILESET_H
// --- end of file ---
